@@ -39,19 +39,18 @@ new_cache = function() {
     # save object names
     x = paste(c(label, objs), collapse = '\t')
     if (file.exists(path)) {
-      lines = readLines(path)
+      lines = read_utf8(path)
       lines = lines[lines != label] # knitr < 1.5 may have lines == label
       idx = substr(lines, 1L, nchar(label) + 1L) == paste0(label, '\t')
       if (any(idx)) {
         lines[idx] = x  # update old objects
       } else lines = c(lines, x)
     } else lines = x
-    writeLines(lines, con = path)
+    write_utf8(lines, path)
   }
-  cache_objects = function(keys, code, label, path) {
+  cache_objects = function(keys, globals, label, path) {
     save_objects(keys, label, valid_path(path, '__objects'))
-    # find globals in code; may not be reliable
-    save_objects(find_globals(code), label, valid_path(path, '__globals'))
+    save_objects(globals, label, valid_path(path, '__globals'))
   }
 
   cache_load = function(hash, lazy = TRUE) {
@@ -78,11 +77,11 @@ new_cache = function() {
     path = valid_path(path, '__packages')
     if (save) {
       x = rev(.packages())
-      if (file.exists(path)) x = setdiff(c(readLines(path), x), .base.pkgs)
-      writeLines(x, path)
+      if (file.exists(path)) x = setdiff(c(read_utf8(path), x), .base.pkgs)
+      write_utf8(x, path)
     } else {
       if (!file.exists(path)) return()
-      for (p in readLines(path))
+      for (p in read_utf8(path))
         suppressPackageStartupMessages(library(p, character.only = TRUE))
     }
   }
@@ -116,6 +115,14 @@ known_globals = c(
   '{', '[', '(', ':', '<-', '=', '+', '-', '*', '/', '%%', '%/%', '%*%', '%o%', '%in%'
 )
 
+# analyze code and find out all possible variables (not necessarily global variables)
+find_symbols = function(code) {
+  if (is.null(code) || length(p <- parse(text = code, keep.source = TRUE)) == 0) return()
+  p = getParseData(p)
+  p = p[p$terminal & p$token %in% c('SYMBOL', 'SYMBOL_FUNCTION_CALL', 'SPECIAL'), ]
+  unique(p$text)
+}
+
 # a variable name to store the metadata object from code chunks
 cache_meta_name = function(hash) sprintf('.%s_meta', hash)
 # a variable name to store the text output of code chunks
@@ -134,7 +141,7 @@ cache_rx = '_[abcdef0123456789]{32}[.](rdb|rdx|RData)$'
 #' object names in these files to automatically build cache dependencies, which
 #' is similar to the effect of the \code{dependson} option. It is supposed to be
 #' used in the first chunk of a document and this chunk must not be cached.
-#' @param path the path to the dependency file
+#' @param path Path to the dependency file.
 #' @return \code{NULL}. The dependencies are built as a side effect.
 #' @note Be cautious about \code{path}: because this function is used in a
 #'   chunk, the working directory when the chunk is evaluated is the directory
@@ -172,7 +179,7 @@ parse_objects = function(path) {
   if (!file.exists(path)) {
     warning('file ', path, ' not found'); return()
   }
-  lines = strsplit(readLines(path), '\t')
+  lines = strsplit(read_utf8(path), '\t')
   if (length(lines) < 2L) return()  # impossible for dependson
   objs = lapply(lines, `[`, -1L)
   names(objs) = lapply(lines, `[`, 1L)
@@ -189,15 +196,15 @@ parse_objects = function(path) {
 #' chunk, which is normally not possible because \pkg{knitr} compiles the
 #' document in a linear fashion, and objects created later cannot be used before
 #' they are created.
-#' @param label the chunk label of the code chunk that has a cache database
-#' @param object the name of the object to be fetched from the database (if
-#'   missing, \code{NULL} is returned)
-#' @param notfound a value to use when the \code{object} cannot be found
-#' @param path the path of the cache database (normally set in the global chunk
-#'   option \code{cache.path})
-#' @param lazy whether to \code{\link{lazyLoad}} the cache database (depending
+#' @param label The chunk label of the code chunk that has a cache database.
+#' @param object The name of the object to be fetched from the database. If it is
+#'   missing, \code{NULL} is returned).
+#' @param notfound A value to use when the \code{object} cannot be found.
+#' @param path Path of the cache database (normally set in the global chunk
+#'   option \code{cache.path}).
+#' @param lazy Whether to \code{\link{lazyLoad}} the cache database (depending
 #'   on the chunk option \code{cache.lazy = TRUE} or \code{FALSE} of that code
-#'   chunk)
+#'   chunk).
 #' @note Apparently this function loads the value of the object from the
 #'   \emph{previous} run of the document, which may be problematic when the
 #'   value of the object becomes different the next time the document is
@@ -277,8 +284,8 @@ rand_seed = quote({
 #' will not be automatically cleaned. You can use this function to identify
 #' these possible files, and clean them if you are sure they are no longer
 #' needed.
-#' @param clean whether to remove the files
-#' @param path the cache path
+#' @param clean Boolean; whether to remove the files.
+#' @param path Path to the cache.
 #' @note  The identification is not guaranteed to be correct, especially when
 #'   multiple documents share the same cache directory. You are recommended to
 #'   call \code{clean_cache(FALSE)} and carefully check the list of files (if
@@ -310,6 +317,6 @@ clean_cache = function(clean = FALSE, path = opts_chunk$get('cache.path')) {
   if (p1 != '') i = i & (substr(base, 1, nchar(p1)) == p1)
   if (!any(i)) return()
   if (clean) unlink(files[i]) else message(
-    'Clean these cache files?\n\n', paste(files[i], collapse = '\n'), '\n'
+    'Clean these cache files?\n\n', one_string(files[i]), '\n'
   )
 }
